TITLE AboutDialog.asm
;                           ====================
;============================  AboutDialog.asm ================================
;==                         ====================                             ==
;==                                                                          ==
;== Project         : ScangII                                                ==
;== Filename        : AboutDialog.asm                                        ==
;== Version         : 1                                                      ==
;== Date            : Jul 15, 2008                                           ==
;== Compiler        : Masm v. 6.14 by Microsoft Corporation.                 ==
;== Purpose         : Handles the about dialog box.							 ==
;== Uses:			: strsafe.lib from Poasm                                 ==
;					  strsafe.inc from                                       ==
;					  http://www.masm32.com/board/index.php?topic=8022.0     ==
;==============================================================================
;==                                                                          ==
;== Copyright (c) 2008, P. H. Duarte                                         ==
;== All rights reserved.                                                     ==
;==                                                                          ==
;==============================================================================

; VERSION HISTORY:
; ----------------

; Version           : 1
; Date              : Jul 15, 2008
; Revised by        : P. H. Duarte
; Description       : Original Version.
; Version			: 1.001
; Date				: Nov 06, 2008
; Description		: * Implemented string safe operations 
;==============================================================================

.686
.model flat, stdcall
option casemap:none
include scangii.inc

;==============================================================================
;==                         Prototypes of local functions                    ==
;==============================================================================

UrlProc PROTO hWin:HWND, uMsg:UINT, wParam: WPARAM, lParam:LPARAM

;==============================================================================
;==                          Local Constants                                 ==
;==============================================================================

.const
DLG_ABOUT			equ 2500	; About dialog box ID.
STC_COPYRIGHT		equ 2501	; Static control for copyright message.
STC_VORTEX_SITE		equ 2502	; Button to open Vortex' site
STC_MASM_JUICE		equ 2503	; Static control for code message.
STC_GET_SCAN		equ 2504	; Static control for message.

;==============================================================================
;==                          Local Data                                      ==
;==============================================================================

.data
szGetScan	CHAR	"Get scan.exe at:", 0

szJuiceSt	CHAR	"-- This is a 100% MASM juice project. --", 13, 10,
					"-- Coded to help coders to code. --", 13, 10,0

szCopyRt	CHAR 	"-- A gui for Vortex' scan --", 13, 10,
					"-- Copyright 2008  ", 169, 13, 10, "-- Paulo H. --", 0
szAppName	CHAR	"ScanGui", 0					
szUrl		CHAR	"www.vortex.masmcode.com", 0 ; Vortex's site.
szHttp		CHAR	"http://", 0
szAbout		CHAR	"About ", 0

.data?
bMouseTouch	BOOL		?	; Stores the mouse movement state.
hOldUrlProc	HINSTANCE	?	; Handle to original DialogBox procedure.
hStcMasm	HANDLE		?	; Static control for code message.
hUrlBrush	HBRUSH		?	; Handle to url static control brush type.
hUrlFontU	HFONT		?	; Handle to underlined font.
hUrlOldFont	HFONT		?	; Handle to original font in url static control.

;==============================================================================

.code

;==============================================================================
;
; Function UrlProc(hWin, uMsg, wParam, lParam ).
;
; Purpose: Make a static control clickable and open internet browser.
;
; Called by:	hDlgAbout. 
;
; Entry:		hWin	= Handle to static link control.
;				uMsg	= Message to process.
;				wParam	= Handle to device context for the static control.
;				lParam	= Handle to static control. 
;
; Description:	Process messages generated by the static control.
;				It changes the color of a clickable text and opens an URL
;				into default web browser through subclassing.	
;
; Return:		FALSE if a message was processed, otherwise 
;				it calls CallWindowProc.
;
; Error Handling:	NA.
;
; Important remark: This function was adapted from RadAsm source 
;					code with kind permission from KetilO.
;============================================================================== 

UrlProc PROC hWin:HWND, uMsg:UINT, wParam: WPARAM, lParam:LPARAM
LOCAL rect:RECT
LOCAL szBuffer[LENGTHOF szHttp + LENGTHOF szUrl + 1]:CHAR
	
	.if dword ptr [uMsg] == WM_SETCURSOR
		invoke LoadCursor, NULL, IDC_HAND
		invoke SetCursor, EAX
		
	.elseif dword ptr [uMsg] == WM_MOUSEMOVE
		invoke GetClientRect, dword ptr [hWin], addr rect
		invoke GetCapture
		
		.if EAX != dword ptr [hWin]
			mov byte ptr ds:[bMouseTouch], TRUE
			invoke SetCapture, dword ptr [hWin]
			invoke SendMessage, dword ptr [hWin], WM_SETFONT, 
			        dword ptr [hUrlFontU], TRUE
		.endif
		
		mov EDX, dword ptr [lParam]
		movzx EAX, DX
		shr EDX, 16
		
		.if EAX > dword ptr [rect.right] || EDX > dword ptr [rect.bottom]
			mov byte ptr ds:[bMouseTouch], FALSE
			invoke ReleaseCapture
			invoke SendMessage, dword ptr [hWin], WM_SETFONT,
			        dword ptr [hUrlOldFont], TRUE
		.endif
		
	.elseif dword ptr [uMsg] == WM_LBUTTONUP
		mov byte ptr ds:[bMouseTouch], FALSE
		invoke ReleaseCapture
		invoke SendMessage, dword ptr [hWin], WM_SETFONT, hUrlOldFont, TRUE
		invoke RtlZeroMemory, addr szBuffer, LENGTHOF szBuffer ; Clear buffer area.
		
		invoke StringCbCat, addr szBuffer, LENGTHOF szBuffer, addr szHttp
		
		invoke StringCbCat, addr szBuffer, LENGTHOF szBuffer, addr szUrl
		lea EBX, dword ptr [szBuffer]
		mov dword ptr [SheInfo.lpFile], EBX
		mov dword ptr [SheInfo.nShow], SW_SHOWMAXIMIZED
		invoke ShellExecuteEx, addr SheInfo
		
	.else
		invoke CallWindowProc, dword ptr [hOldUrlProc], dword ptr [hWin], 
		        dword ptr [uMsg], dword ptr [wParam], dword ptr [lParam]
		ret
	.endif
	mov EAX, FALSE
	ret
	
UrlProc endp
	
;===============================================================================
;
; DlgProc (hDlgAbout, uMsg, wParam, lParam)
;
; Purpose:	To show an About dialog box with informations about the program.
;
; Called by: WndProc
;
; Entry:	hDlgAbout	=	Handle to dialog box.
;			uMsg		=	Message to process.
;			wParam		=	Additional parameter.
;			lParam		=	Additional parameter.
;
; Return:	TRUE		=	A message was processed.
;			FALSE		=	No message was processed.
;=================================================================================		

DlgProc PROC hDlgAbout:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM
LOCAL 	hItem:HANDLE
LOCAL 	hDc:HDC
LOCAL 	lf:LOGFONT
LOCAL	szBuffer[LENGTHOF szAbout + LENGTHOF szAppName + 1]:CHAR

	.if uMsg == WM_INITDIALOG
		invoke RtlZeroMemory, addr szBuffer, LENGTHOF szBuffer
		invoke StringCbCat, addr szBuffer, LENGTHOF szBuffer, addr szAbout

		invoke StringCbCat, addr szBuffer, LENGTHOF szBuffer, addr szAppName
		invoke SetWindowText, dword ptr [hDlgAbout], addr szBuffer
		
	; --> Write on static controls. <-- 		
		
		invoke SendDlgItemMessage, dword ptr [hDlgAbout], STC_MASM_JUICE, 
				WM_SETTEXT, 0, addr szJuiceSt
		
		invoke SendDlgItemMessage, dword ptr [hDlgAbout], STC_COPYRIGHT, 
				WM_SETTEXT, 0, addr szCopyRt
		
		invoke SendDlgItemMessage, dword ptr [hDlgAbout], STC_GET_SCAN, 
				WM_SETTEXT, 0, addr szGetScan
		
		invoke GetDlgItem, dword ptr [hDlgAbout], STC_VORTEX_SITE
		mov dword ptr [hItem], EAX
		
		invoke SendMessage, dword ptr [hItem], WM_GETFONT, 0, 0
		mov dword ptr [hUrlOldFont], EAX
		
		invoke GetObject, dword ptr [hUrlOldFont], SIZEOF LOGFONT, addr lf
		mov dword ptr [lf.lfUnderline], TRUE
		
		invoke CreateFontIndirect, addr lf
		mov dword ptr [hUrlFontU], EAX
		
		invoke SendDlgItemMessage, dword ptr [hDlgAbout], STC_VORTEX_SITE, 
				WM_SETTEXT, 0, addr szUrl
		
		invoke SetWindowLong, dword ptr [hItem], GWL_WNDPROC, addr UrlProc
		mov dword ptr [hOldUrlProc], EAX
		
		invoke GetSysColor, COLOR_3DFACE
		invoke CreateSolidBrush, EAX
		mov dword ptr [hUrlBrush], EAX
		
	.elseif uMsg == WM_CTLCOLORSTATIC
		push dword ptr [lParam]	; Handle to static control.
		pop dword ptr [hItem]
		
		push dword ptr [wParam]	; wParam -> Handle to device context for
								; static control window.
		pop dword ptr [hDc]
		
		invoke GetDlgItem, dword ptr [hDlgAbout], STC_VORTEX_SITE
		mov ECX, EAX
		xor EAX, EAX
		
		.if ECX == dword ptr [hItem]
			
			.if byte ptr ds:[bMouseTouch] != FALSE
				mov EAX, 0FF0000h	; This color is Blue.
			.endif			
			
			invoke SetTextColor, dword ptr [hDc], EAX ; Changes the text color.
			invoke SetBkMode, dword ptr [hDc], TRANSPARENT
			mov eax, dword ptr [hUrlBrush]
			
		.endif
		ret
		
	.elseif uMsg == WM_CLOSE
		invoke DeleteObject, dword ptr [hUrlFontU]
		invoke DeleteObject, dword ptr [hUrlBrush]
		invoke EndDialog, dword ptr [hDlgAbout], FALSE
		
	.else
		mov EAX, FALSE
		ret
		    
	.endif
	mov EAX, TRUE
	ret
	
DlgProc ENDP
end
;==============================================================================
;==                                 EOF                                      ==
;==============================================================================