TITLE Choose.asm
;                           ======================
;============================      InOut.asm     ==============================
;==                         ======================                           ==
;==                                                                          ==
;== Project         : Scangui                                                ==
;== Filename        : Choose.asm                                             ==
;== Version         : 1                                                      ==
;== Date            : Jul 15, 2008                                           ==
;== Compiler        : Masm v. 6.14 by Microsoft Corporation.                 ==
;== Purpose         : Execute the operation of input and output like         ==
;==                   choose a file to be scanned, set the path for scan.exe,==
;==                   open a .txt file to be read, save the path for         ==
;==                   scan.exe into a file named settings.ini and read it    ==
;==                   back, run scan.exe with correct parameters.            ==
;==                                                                          ==
;==============================================================================
;==                                                                          ==
;== Copyright (c) 2008, P. H. Duarte                                         ==
;== All rights reserved.                                                     ==
;==                                                                          ==
;==============================================================================

; VERSION HISTORY:
; ----------------
; Version           : 1
; Date              : Jul 15, 2008
; Revised by        : P. H. Duarte
; Description       : Original Version.
;==============================================================================
; Version           : 2.01
; Date              : nov 15, 2008
; Revised by        : P. H. Duarte
; Description       : * Implemented a function to set the path for scan.exe.
;                     * Implemented a function to edit a txt file.
;                     * Implemented a function to save the path for scan.exe.
;                     * Implemented a function to read settings.ini.
;                     * Added the procedure which runs scan.exe.
;==============================================================================

.686
.model flat, stdcall
option casemap:none

;==============================================================================
;==                        Include Files                                     ==
;==============================================================================
include ScanGII.inc

;==============================================================================
;==                         Prototypes of local functions                    ==
;==============================================================================

CreateIniFile   PROTO   hwnd:HWND

;==============================================================================
;==                          Local Data                                      ==
;==============================================================================
.const
uFlagsOfn		        equ	OFN_PATHMUSTEXIST + OFN_LONGNAMES +\
                            OFN_READONLY + OFN_EXPLORER

uFlagsBinfo		        equ BIF_DONTGOBELOWDOMAIN +	BIF_EDITBOX + \
                            BIF_NEWDIALOGSTYLE + BIF_NONEWFOLDERBUTTON + \ 
                            BIF_USENEWUI

SEE_MASK_FLAG_LOG_USAGE	equ 04000000h

uSeeMask				equ SEE_MASK_FLAG_LOG_USAGE

cMaxLen					equ 1024

.data

; --> OpenFileName data <--

szImpExt			CHAR	"imp", 0	; Default filename extension.
szFilter 			CHAR	"Assembly (*.asm)", 0, "*.asm", 0
					CHAR	"Include (*.inc)", 0, "*.inc", 0
 					CHAR	"All files", 0, "*.*", 0, 0
lpstrFile			CHAR	MAX_PATH 	DUP (0) ; Buffer for source file name.
lpstrFileTitle		CHAR	MAX_PATH	DUP (0)
szScanPath			CHAR	MAX_PATH	DUP	(0) ; Scan.exe path. 

szTitle				CHAR	"Select file to scan:", 0   ; OpenFileName title.
szOperation 	    CHAR	"open", 0   ; Operation to execute.
szBrowseTitle		CHAR	"Choose scan.exe path:", 0
szSlash             CHAR    "\",0
szIniFile           CHAR    "Settings.ini",0
szIniWriteError     CHAR    "It is not possible to create Settings.ini.",13,10,
                            "Try again."
szHaveNotSelected	CHAR 	"You didn't selected a file to be scanned.", 0
szSetPathFirst      CHAR    "Set scan.exe path, please.", 0
szIniFindError      CHAR    "The file settings.ini can't be used or",13,10,
                            "can't be found.",0
szInvalidName		CHAR	"Invalid name or path name.",0
szScanNotFound		CHAR	"Scan.exe not found in this directory.", 0
szPipeNotAllowed	CHAR	"It is not possible to create a communication",
                             13,10, "between ScanGui and Scan.exe.", 0
szProcessNotCreated	CHAR	"It is not possible execute Scan.exe.",13,10,
                            "Check app path.", 0
szFailedToCreate	CHAR	"It is not possible to create the imports file.",
                             13,10, "Try again.", 0
szScanError			CHAR	"Scan.exe detected some errors.", 13,10,
                            "See warnings panel.", 0
szSucess  			CHAR	"The file was successfully generated.", 0

brInfo	BROWSEINFO 		    <NULL, NULL, OFFSET szScanPath,\
                            OFFSET szBrowseTitle,\
                            uFlagsBinfo, NULL, NULL, 0>

ofn		OPENFILENAME	    <SIZEOF OPENFILENAME, NULL, NULL, OFFSET szFilter, \
		  					NULL, NULL, NULL, OFFSET lpstrFile, MAX_PATH, \
		  					OFFSET lpstrFileTitle, MAX_PATH, NULL, OFFSET szTitle, \
		  					uFlagsOfn, NULL, NULL, NULL, NULL, NULL,NULL>
		  					
SheInfo	SHELLEXECUTEINFO	<SIZEOF SHELLEXECUTEINFO, uSeeMask, NULL,\
                            OFFSET szOperation, NULL, NULL, NULL,\
                            SW_SHOW, OFFSET hInstApp, NULL, NULL,\
                            NULL, NULL, <NULL>, OFFSET hNewProcess>

                            
sat		  		SECURITY_ATTRIBUTES <SIZEOF SECURITY_ATTRIBUTES, NULL, TRUE>

.data?
hInstApp	HINSTANCE	?	; Handle to app executed.
hNewProcess HANDLE		?   ; Handle to new process created.

;pidl        DWORD   ?
StartInfo 	STARTUPINFO			<?>
ProcInfo  	PROCESS_INFORMATION	<?>

;==============================================================================		

.code

;==============================================================================
;
; ChooseFile (hwnd)
;
; Purpose:	Choose the source file to be scanned.
;
; Called by: WndProc
;
; Entry:	hwnd  = Handle to main window.
;
; Return:   0 = No error occured.
;           1 = An error occured.
;			lpstrFileTitle  = The name of source file.
;			szOutName		= Name of importations file.
;
; Error handling: Verifies if exists a valid path and a valid name 
;                 for a source file. It shows a message box wich asks 
;                 the user to choose a file with valid path and valid name.
;==============================================================================

ChooseFile proc hwnd:HWND
	
	push dword ptr [hwnd]
	pop dword ptr [ofn.hwndOwner]
	invoke GetOpenFileName, OFFSET ofn
	.if EAX == 0	; No file to scan
		invoke MsgBoxProc, dword ptr [hwnd], addr szHaveNotSelected, 
				addr szError, addr szClose, MB_ICONSTOP
		mov EAX, 01
	.else
		mov ECX, MAX_PATH 				 
		mov EDI, OFFSET szOutName 		;	EDI -> File of imported functions.
		push EDI 						 
		mov ESI, OFFSET lpstrFileTitle 	;	ESI -> Name of source file.
		
		cld								; Forward searching
		rep movsb						
		
	; --> Changes the file name extension. <--
		
		pop EDI 					
		mov AL, "." 					;	Al  -> Holds the delimiter character.
		and EAX, 000000FFh				;	Only '.' is important.				 
		.repeat
			inc EDI 
			.if byte ptr ds:[EDI] == 0
				invoke MsgBoxProc, dword ptr [hwnd], addr szInvalidName, 
						addr szError, addr szClose, MB_OK
				mov EAX, 01
				ret
			.endif
		.until byte ptr ds:[EDI] == AL	
		inc EDI 				        ;	EDI -> '.'.
		
		mov ESI, OFFSET szImpExt		;	ESI -> "imp". 		
		mov ECX, 03
		cld
		rep movsb
		xor EAX, EAX
	.endif

	ret
	
ChooseFile endp

;==============================================================================
;
; SetScanPath (hwnd)
;
; Purpose:	Set path for scan.exe
; 
; Called by: WndProc
;
; Entry:	hwnd  = Handle to main window.
;
; Return:	No meaningful values returned.
;           			
;==============================================================================

SetScanPath proc hwnd:HWND 
LOCAL pidl:DWORD

	invoke CoInitializeEx, NULL, COINIT_APARTMENTTHREADED
		
	invoke SHBrowseForFolder, OFFSET brInfo
	mov dword ptr [pidl], EAX
	
	; --> Copy scan.exe path into szScanPath <--
	
	invoke RtlZeroMemory, addr szScanPath, MAX_PATH
	
	invoke SHGetPathFromIDList, dword ptr [pidl], 
	        addr szScanPath
	.if EAX == FALSE
	    invoke MsgBoxProc, dword ptr [hwnd], addr szSetPathFirst, addr szError,
	            addr szClose, MB_OK
	    ret
	.endif
	
	invoke StringCbCat,addr szScanPath, 
	        MAX_PATH, addr szSlash
	        
	; --> Once scan.exe path is ok create a file to store it. <--
	invoke CreateIniFile, dword ptr [hwnd]
	
	invoke CoTaskMemFree, dword ptr [pidl]
	invoke CoUninitialize
	
	ret
	
SetScanPath endp

;==============================================================================
;
; CreateIniFile (hwnd)
;
; Purpose:	Create settings.ini which contains the full path for scan.exe.
; 
; Called by: SetScanPath
;
; Entry:	hwnd  = Handle to main window.
;
; Return:	No signifcant values.
;
; Error handling: Verify the scan.exe path and shows a message box
;                 which tells the user about this error. 
;                 Verify the creation of settings.ini and shows a message
;                 box which tells the user about this error.
;                 In both cases the program can operate without problems.
;==============================================================================

CreateIniFile proc hwnd:HWND
	
LOCAL hFile:HANDLE ; Handle to file settings.ini.
LOCAL iBytesToWrite:DWORD ; Number of bytes to write on Settings.ini.
LOCAL piWrittenBytes:DWORD; Number of written bytes on Settings.ini. 
	
	; --> Verify if exists a non null string to szScanPath. <--
	
	invoke StringCbLength, addr szScanPath, MAX_PATH, addr iBytesToWrite
	
	.if dword ptr [iBytesToWrite] == 0
		invoke MsgBoxProc, dword ptr [hwnd], addr szSetPathFirst, addr szError,
		        addr szClose, MB_ICONWARNING
		ret
	.endif
	
	
	invoke CreateFile, addr szIniFile,GENERIC_WRITE, FILE_SHARE_WRITE,
	        NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL
	
	.if EAX == INVALID_HANDLE_VALUE
		invoke MsgBoxProc, dword ptr[hwnd], addr szIniWriteError, 
		addr szError, addr szClose, MB_ICONWARNING
		ret
	.endif
	
	mov dword ptr [hFile], EAX
	
	; --> Start point is the beginning of the file. <--
	invoke SetFilePointer, dword ptr [hFile], 0, NULL, FILE_BEGIN
	
	.if EAX == INVALID_SET_FILE_POINTER
		invoke MsgBoxProc, dword ptr [hwnd], addr szIniWriteError,
		        addr szError, addr szClose, MB_ICONWARNING
		ret
	.endif 
	
	; --> Saves Scan.exe path. <--
	invoke WriteFile, dword ptr [hFile], addr szScanPath, 
	        dword ptr [iBytesToWrite], addr piWrittenBytes, NULL
	
	invoke CloseHandle, dword ptr [hFile]
	ret
	
CreateIniFile endp

;==============================================================================
;
; LoadIniFile (hwnd)
;
; Purpose:	Load settings.ini which contains the full path for scan.exe.
; 
; Called by:     Verify
;
; Entry:	     hwnd  = Handle to main window.
;
; Return:	      0 = No error ocurred.
;                 1 = Settings.ini was not found or can't be opened.
;
; Error handling: Verifies if settings.ini exists and try to open it.
;                 If the file can't be opened or it doesn't exit, a message
;                 box explains this to user.
;==============================================================================
	
LoadIniFile proc hwnd:HWND
LOCAL hFile:HANDLE
LOCAL iBytesToRead:DWORD
LOCAL piReadBytes:DWORD
	
	invoke CreateFile, addr szIniFile, GENERIC_READ, FILE_SHARE_READ,
	        NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL
	.if EAX == INVALID_HANDLE_VALUE || EAX == 0
		invoke MsgBoxProc, dword ptr [hwnd], addr szIniFindError,
        		addr szError, addr szClose, MB_ICONWARNING
        mov EAX, 1
		ret
	.endif

	mov dword ptr [hFile], EAX
	
	invoke SetFilePointer, dword ptr [hFile], 0, NULL, FILE_BEGIN
	
	.if EAX == INVALID_SET_FILE_POINTER
		invoke MsgBoxProc, dword ptr [hwnd], addr szIniFindError,
		        addr szError, addr szClose, MB_ICONWARNING
        mov EAX, 1
		ret
	.endif

	mov dword ptr [iBytesToRead], MAX_PATH
	invoke RtlZeroMemory, addr szScanPath, MAX_PATH
	invoke ReadFile, dword ptr [hFile], addr szScanPath,
	        dword ptr [iBytesToRead], addr piReadBytes, NULL
    
	invoke CloseHandle, dword ptr [hFile]
    
    xor EAX, EAX
	ret
	
LoadIniFile endp
	
;==============================================================================
;
; OpenToRead (hwnd, pszFileToOpen)
;
; Purpose:	 Open a file with the default text editor.
; 
; Called by: WndProc
;
; Entry:	hwnd          = Handle to main window.
;           pszFileToOpen = Pointer to file name string.
;
; Return:	No meaningful values returned.
;           			
; Error handling: NA.
;==============================================================================

OpenToRead proc hwnd:HWND, pszFileToOpen:LPSTR
	
	push dword ptr [pszFileToOpen]
	pop  dword ptr [SheInfo.lpFile]
	invoke lstrlen, addr szScanPath
	.if EAX !=0
		invoke SetCurrentDirectory, addr szScanPath
	.else
		invoke SetCurrentDirectory, addr szCurrentDir
	.endif
	invoke ShellExecuteEx, addr SheInfo
	
	ret
OpenToRead endp

;==============================================================================
;
; Executa (hwnd, lpLine)
;
; Purpose:	Execute scan.exe.
;
; Entry:	hwnd = Handle to main program window.
;           lpLine = Pointer to command line to scan.exe.
; Called by: WndProc
;
; Error Handling: Verify the pipe creation and the new process creation.
;                 In both cases it shows a message box which tells the user
;                 the error. For the first case this function calls clean
;                 and restart the main dialog box.
;                 It also verifies any message from scan.exe and
;                 place them at warnings panel (EDT_3).
;
; Return:	No meaningful values returned
;
; Important remark: I followed Iczellion's tutorial n. 21: PIPE.
;==============================================================================

Executa proc hwnd:HWND, lpLine:LPSTR
	
LOCAL hReadPipe:DWORD			;	Handle to read end of the pipe.
LOCAL hWritePipe:DWORD			;	Handle to write end of the pipe.
LOCAL iBytesRead:DWORD			;   Number of bytes read.
LOCAL hWarn:DWORD				;	Handle to EDT_3 (warning messages).
LOCAL szbuffer[cMaxLen]:CHAR	; 	Holds pipe messages.
	
	invoke GetDlgItem, dword ptr [hwnd], EDT_3
	mov dword ptr [hWarn], EAX
	invoke SetDlgItemText, dword ptr [hwnd], EDT_3, NULL ; Clear warnings.
	
	; --> Create pipeline to communicate with scan.exe. <--
	
	invoke CreatePipe, addr hReadPipe, addr hWritePipe, addr sat, 0
	.if EAX == 0				;	Pipe is not created.
		invoke MsgBoxProc, dword ptr [hwnd], addr szPipeNotAllowed, 
				addr szError, addr szClose, MB_OK
		invoke Clean, dword ptr [hwnd]
		ret
	.endif
	
	; --> Initialize structure StartInfo. <-- 
	
	mov	dword ptr [StartInfo.cb], SIZEOF STARTUPINFO
	invoke GetStartupInfo, OFFSET StartInfo
	mov EAX, dword ptr  dword ptr [hWritePipe]
	mov dword ptr [StartInfo.hStdOutput], EAX
	mov dword ptr [StartInfo.hStdError], EAX
	mov dword ptr [StartInfo.dwFlags], STARTF_USESHOWWINDOW + \
	                                   STARTF_USESTDHANDLES
	mov dword ptr [StartInfo.wShowWindow], SW_HIDE
	
	; --> Run Scan.exe. <--
	
	invoke CreateProcess, NULL, dword ptr [lpLine], NULL, NULL, 
			TRUE, HIGH_PRIORITY_CLASS, NULL, NULL, addr StartInfo, 
			addr ProcInfo
	
	; --> Error to create process. <-- 
	
	.if EAX == 0
		invoke MsgBoxProc, dword ptr [hwnd], addr szProcessNotCreated, 
				addr szError, addr szClose, MB_OK
		ret
	.else
		invoke CloseHandle, dword ptr [ProcInfo.hThread]
		invoke CloseHandle, dword ptr [ProcInfo.hProcess]
		invoke CloseHandle, dword ptr [hWritePipe]
		.while TRUE
			invoke RtlZeroMemory, addr szbuffer, cMaxLen
			invoke ReadFile, dword ptr [hReadPipe], addr szbuffer,
					LENGTHOF szbuffer - 1, addr iBytesRead, NULL
			.break .if (!EAX)
			
	; --> Show error messages from scan.exe. <--
			
			invoke SendMessage, dword ptr [hWarn], EM_SETSEL, -1, 0
			invoke SendMessage, dword ptr [hWarn], EM_REPLACESEL, \
								FALSE, addr szbuffer
		.endw
	.endif
	invoke CloseHandle, dword ptr [hReadPipe]
	invoke GetDlgItemText, dword ptr [hwnd], EDT_3, addr szbuffer, MAX_PATH	
	.if EAX == 0						;	Succesfull operation.
		invoke MessageBox,NULL, addr szSucess, 
				addr szAppName, MB_ICONINFORMATION 
	.else								;	Scan.exe found an error.
		invoke MsgBoxProc, dword ptr [hwnd], addr szScanError, 
				addr szError, addr szClose, MB_OK
	.endif
	
	ret
Executa endp

end		
;==============================================================================
;==                                 EOF                                      ==
;==============================================================================