;                           ====================
;============================  Messbox.asm     ================================
;==                         ====================                             ==
;==                                                                          ==
;== Project         : ScanGui                                                ==
;== Filename        : Messagier.asm                                          ==
;== Version         : 1                                                      ==
;== Date            : Jul 15, 2008                                           ==
;== Compiler        : Masm v. 6.14 by Microsoft Corporation.                 ==
;== Purpose         : Show message box with information to user.             ==
;==                                                                          ==
;==============================================================================
;==                                                                          ==
;== Copyright (c) 2008, P. H. Duarte                                         ==
;== All rights reserved.                                                     ==
;==                                                                          ==
;==============================================================================

; VERSION HISTORY:
; ----------------

; Version           : 2
; Date              : Nov 10, 2008
; Revised by        : P. H. Duarte
; Description       : * Implemented a custom dialog box.
;==============================================================================

.686
.model flat, stdcall
option casemap:none

;==============================================================================
;==                         Include Files                                    ==
;==============================================================================

include ScanGII.inc

;==============================================================================
;==                         Prototypes of local functions                    ==
;==============================================================================

CBTProc		PROTO	nCode:DWORD, wParam:WPARAM, lParam:LPARAM

;==============================================================================
;==                          Local Data                                      ==
;==============================================================================

.data
szHaveToSelect		CHAR 	"You have to select a file or type in ",
					        "a filename first.", 0
szFileNotFound		CHAR	"The scan manual is not in this folder.", 0
szAcessDenied		CHAR	"The operating system denied access to file.", 0
szNotAssociated		CHAR	"There's not a default app to open .txt files.",0
szClose				CHAR	"Close",0

.data?
dwMsHook			DWORD	?
lpszBtnText			LPSTR	?

;==============================================================================

.code

;==============================================================================
;==                                                                          ==
;== Function CBTProc(nCode, wParam, lParam)                                  ==
;==                                                                          ==
;== Purpose: Traps message boxes and customize their buttons.                ==
;==                                                                          ==
;== Entry:     nCode  : Determines how the hook  process the messages.       ==  
;==            wParam : Specifies the handle to the new window.              ==
;==            lParam : Specifies a pointer to a CBT_CREATEWND structure.    ==
;==                                                                          ==
;== Called by: MsgBoxProc                                                    ==
;==                                                                          ==
;== Description:                                                             ==
;==                                                                          ==
;== Returns: 0 : The message box was customized. (Desired hook)              ==
;==          CallNextHookEx: Default condition.                              ==
;==                                                                          ==
;== Error Handling: NA                                                       ==
;==                                                                          ==
;==============================================================================

CBTProc proc nCode:DWORD, wParam:WPARAM, lParam:LPARAM
LOCAL hwndButton:HANDLE
	
	.if dword ptr [nCode] < 0
		invoke CallNextHookEx, dword ptr [dwMsHook], dword ptr [nCode], 
				dword ptr [wParam], dword ptr [lParam]
		ret
	.elseif dword ptr [nCode] == HCBT_ACTIVATE
		invoke GetDlgItem, dword ptr [wParam], IDOK
		mov dword ptr [hwndButton], EAX
		invoke SetWindowText, dword ptr [hwndButton], dword ptr [lpszBtnText]
	.endif
	xor EAX, EAX
	ret
CBTProc endp

;==============================================================================
;==                                                                          ==
;== Function (hwnd, pszMessage, pszCaption, pszButtonText, uType )           ==
;==                                                                          ==
;== Purpose: To show a message box.                                          ==
;==                                                                          ==
;== Entry:  hwnd = Handle to main window.                                    ==
;==         pszMessage = Pointer to string that holds the box message.       ==
;==         pszCaption = Pointer to string that holds the box title.         ==
;==         pszButtonText = Pointer to string that holds the button text.    ==
;==         uType = The type of message box.                                 ==
;==                                                                          ==
;== Called by: WndProc, Verify, Executa, Montalinha, StartCombo              ==
;==                                                                          ==
;== Description: Show a message box.                                         ==
;==                                                                          ==
;== Returns: No meaningful values.                                           ==
;==                                                                          ==
;== Error Handling: NA.                                                      ==
;==                                                                          ==
;==============================================================================	
MsgBoxProc proc hwnd:HANDLE, pszMessage:LPSTR, pszCaption:LPSTR, 
				pszButtonText:LPSTR, uType:UINT
	
	push OFFSET szClose
	pop  dword ptr [lpszBtnText]
	
	invoke GetCurrentThreadId
	
	invoke SetWindowsHookEx, WH_CBT, addr CBTProc, NULL, EAX
	mov dword ptr [dwMsHook], EAX
	
	invoke MessageBox, dword ptr [hwnd], dword ptr [pszMessage],
	        dword ptr [pszCaption], dword ptr [uType]
	invoke UnhookWindowsHookEx, dword ptr [dwMsHook]
	
	ret
	
MsgBoxProc endp
end
;==============================================================================
;==                                 EOF                                      ==
;==============================================================================