TITLE ScanGII.asm
;                             ====================
;==============================   ScanGII.asm    ==============================
;==                           ====================                           ==
;==                                                                          ==
;== Project         : ScanGui                                                ==
;== Filename        : ScanGuII.asm                                           ==
;== Version         : 2                                                      ==
;== Date            : Jul 15, 2008                                           ==
;== Compiler        : Masm v. 6.14 by Microsoft Corporation.                 ==
;== Purpose         : This is a GUI for Vortex' Scan tool                    ==
;== Restrictions	: It uses strsafe.lib from pelles and strsafe.inc.  	 ==
;==                   Both files can be found here:                          ==
;==                                                                          ==
;==    http://www.masm32.com/board/index.php?topic=8022.msg59040#msg59040    ==
;==============================================================================
;==                                                                          ==
;== Copyright (c) 2008, P. H. Duarte                                         ==
;== All rights reserved.                                                     ==
;==                                                                          ==
;==============================================================================
; VERSION HISTORY:
; ----------------
;==============================================================================
; Version           : 2
; Date              : Jul 15, 2008
; Revised by        : P. H. Duarte
; Description       : * Changed the main dialog box design.
;                     * New About Dialog Box added.
;                     * There was a bug when scan.exe was executed. There is
;						 no such bug anymore!
;					  * Modularized design of program.
;					  * Created 03 methods of compilation ( described below ).
;==============================================================================
; Version			: 2.01
; Date				: Nov 10, 2008
; Description		: * Implemented custom message boxes.
;					  * Removed string table.
;					  * Added new error messages.
;					  * Used string safe functions from strsafe.h 
;						and strsafe.lib.
;					  * Implemented dialog to choose Scan.exe path.
;                     * Use of externdef instead of public and extern.
;                     * Implemented code to create a file and store the
;                       path for scan.exe.
;                     * Removed some useless error messages.
;                     * Implemented a custom message box.
;                     * Removed some modules and concatenate code better.
;==============================================================================
;How to compile:
;
;	1 - For RadAsm users:
;		    If you use RadAsm open the ScanGII.rap click build modules on make
;           menu and then click build on the same menu.
;           The executable is ScangII.exe
;
;	2 - For those ones who don't use RadAsm:
;		    Use the nmakemakefile with NMAKE like this:
;			    NMAKE 
;
;	3 - Or you can run build.bat:
;			build.bat
;
;   4 - Or you can link the .obj files: (see obj folder)
;            c:\masm32\bin\LINK.EXE /SUBSYSTEM:WINDOWS /VERSION:4.0 /RELEASE
;            /ENTRY:start /OPT:REF /LIBPATH:"c:\masm32\lib" 
;            /OUT:"scan.exe"*.OBJ scangII.res	
;
;	All of them assume this folder structure:
;		C:\masm32
;==============================================================================

; The module AboutDialog.asm has been coded with part of RadAsm code with
; kind permission of KetilO!

;==============================================================================

.686
.model flat, stdcall
option casemap :none

;==============================================================================
;=                         Include Files                                      =
;==============================================================================

include	ScanGII.inc

;==============================================================================
;=                Prototypes of local functions                               =
;==============================================================================

WinMain   PROTO	:HINSTANCE, :HINSTANCE, :LPSTR,  :DWORD
WndProc   PROTO	:HWND,      :UINT,      :WPARAM, :LPARAM

;==============================================================================
;=                           Local Data                                       =
;==============================================================================

.const
;SCanIIMenu.mnu
IDR_MENU			    equ 10000
IDM_FILE			    equ 10001
IDM_SCAN_PATH		    equ 10003
IDM_CHOOSE_FILE		    equ 10002
IDM_EXIT			    equ 10004
IDM_EDIT			    equ 10005
IDM_EDIT_SCAN_INI	    equ 10006
IDM_RESET			    equ 10007
IDM_HELP			    equ 10008
IDM_ABOUT			    equ 10009
IDM_READ_ME			    equ 10010
                        
;Res\ScanGIIAcl.rc      
AccelTable			    equ 2000
VK_IDM_EXIT			    equ 2020
VK_IDM_EDIT_SCAN_INI    equ 2025
VK_IDM_SCAN_PATH	    equ 2030
VK_IDM_RESET		    equ 2035
VK_IDM_READ_ME		    equ 2045
VK_IDM_CHOOSE_FILE	    equ 2050
                        
;Res\ScanGIIRes.rc
IDI_MAIN				equ 200

;==============================================================================

.data?
hCheck		HANDLE 		?	; Handle to check box controls.
hCombo		HANDLE 		?	; Handle to combo box.
hCursor		HANDLE 		?	; Handle to cursor.
hDlg		DWORD		?	; Handle to main dialog box.
hInstance	HINSTANCE 	?	; Handle to current instance of the application
lpCmdLine	LPSTR		?	; Pointer to app command line.

                	
.data           	
szAppMenuName		CHAR 	"ScanGuiMenu", 0
szAppClassName		CHAR 	"DLGCLASS", 0
szAttention			CHAR 	"Attention!", 0	; Title for attention msg boxes.
szError				CHAR	"Error", 0		; Title for error msg boxes.
szDlgAbout			CHAR	"DLG_ABOUT", 0
szDlgMain			CHAR	"DLG_MAIN", 0
szReadMe		    CHAR	"Readme.txt", 0
szScanini			CHAR	"scan.ini", 0
szDialogFailed		CHAR	"Main dialog creation failure.Click OK to end ",
							"the program.", 0
szWindowVerError	CHAR	"This program can't run in this", 
							"Windows version.", 0
szMessagesError		CHAR	"Windows messages error.Click OK to ",
							"end the program.", 0

szOutName			CHAR	MAX_PATH	DUP (0) ; Importations file name.
szCurrentDir		CHAR	MAX_PATH	DUP (0) ; Current folder name.

;==============================================================================
;=                           Local Structures                                 =
;==============================================================================

icex	  	INITCOMMONCONTROLSEX <SIZEOF INITCOMMONCONTROLSEX, \
									ICC_USEREX_CLASSES +\
									ICC_UPDOWN_CLASS + \
									ICC_HOTKEY_CLASS + \
									ICC_STANDARD_CLASSES>
                                
;==============================================================================

.code
start:

	invoke GetModuleHandle, NULL
	mov dword ptr [hInstance], EAX
	invoke InitCommonControlsEx, addr icex
	invoke GetCommandLine
	mov dword ptr [lpCmdLine], EAX ; Extract the current directory.
	inc EAX
	push EAX
	invoke lstrlen, EAX
	std
	pop EDI
	push EDI
	add EDI, EAX
	xchg EAX, ECX
	mov AL, "\"
	repne scasb
	pop EDI
	add ECX, 3						; ECX -> 03 positions less than we need.
	cld								; For Windows API's work correctly.
	invoke lstrcpyn, addr szCurrentDir, EDI, ECX
	invoke WinMain, dword ptr [hInstance], NULL, dword ptr [lpCmdLine],
			SW_SHOWDEFAULT
	invoke ExitProcess, 0
	ret

;==============================================================================
;
; WinMain (hInst, hPrevInst, CmdLine, CmdShow)
;
; Purpose:	Program entry point.
;
; Entry:	hInst		=	Handle to current instance of the application.
;			hPrevInst	=	Handle to app previous instance.
;			CmdLine		=	Pointer to a null terminated string specifying 
;							app command line.
;			CmdShow		=	Specifies how the window is to be shown.
;
; Error Handling:		= 	Quit the program if the main window can not be
;							registered or when the main dialog box could 
;							not be initialized.
;
; Return:	TRUE	    =	A message was processed.
;			FALSE	    =	No message was processed;
;						 	Window class registration failed;
;						 	The main dialog box could not be initialized. 
;==============================================================================
	
WinMain PROC hInst:HINSTANCE, hPrevInst:HINSTANCE, CmdLine:LPSTR, CmdShow:DWORD
LOCAL wc:WNDCLASSEX
LOCAL msg:MSG
LOCAL hAccel:HACCEL	
		
	mov dword ptr [wc.cbSize], SIZEOF WNDCLASSEX
	mov dword ptr [wc.style], CS_HREDRAW or CS_VREDRAW
	mov dword ptr [wc.lpfnWndProc], OFFSET WndProc
	mov dword ptr [wc.cbClsExtra], 0
	mov dword ptr [wc.cbWndExtra], DLGWINDOWEXTRA
	push dword ptr [hInst]
	pop dword ptr [wc.hInstance]
	mov dword ptr [wc.hbrBackground], COLOR_BTNFACE+1
	mov dword ptr [wc.lpszMenuName], IDR_MENU
	mov dword ptr [wc.lpszClassName], OFFSET szAppClassName
	invoke LoadIcon, hInst, IDI_MAIN
	mov dword ptr [wc.hIcon], EAX
	mov dword ptr [wc.hIconSm], NULL
	invoke LoadCursor, NULL, IDC_ARROW
	mov dword ptr [wc.hCursor], EAX
	invoke LoadCursor, NULL, IDC_HAND
	mov dword ptr [hCursor], EAX
	invoke RegisterClassEx, addr wc
	
	.if (EAX == 0)								; Class registering error.
		invoke MsgBoxProc, NULL, addr szWindowVerError,	addr szError,
				addr szClose, MB_OK
		ret
	.endif

	invoke CreateDialogParam, dword ptr [hInstance], addr szDlgMain,
				NULL, addr WndProc,	0
	
	.if (EAX == NULL)							; Main dialog error.
		invoke MsgBoxProc, NULL, addr szDialogFailed, addr szError,
				addr szClose, MB_OK
		ret
	.endif
	
	mov dword ptr [hDlg], EAX
	invoke ShowWindow, dword ptr [hDlg], SW_SHOWNORMAL
	invoke UpdateWindow, dword ptr [hDlg]
	invoke GetDlgItem, dword ptr [hDlg], EDT_1
	invoke SetFocus, EAX

	invoke LoadAccelerators, dword ptr [hInst], AccelTable
	mov dword ptr [hAccel], EAX
	
	.while TRUE
		invoke GetMessage, addr msg, NULL, 0, 0
		.break .if ( EAX == 0)
		
		.if sdword ptr EAX == -1
			invoke MsgBoxProc, NULL, addr szMessagesError, addr szError,
					addr szClose, MB_OK
			invoke ExitProcess, 0
		.endif
		
		invoke TranslateAccelerator, dword ptr [hDlg], 
				dword ptr [hAccel], addr msg
		
		.if !EAX
			invoke IsDialogMessage, dword ptr [hDlg], addr msg
			
			.if !EAX
				invoke TranslateMessage, addr msg
				invoke DispatchMessage, addr msg
			
			.endif
		
		.endif
	
	.endw
	
	invoke UnregisterClass, OFFSET szAppClassName, dword ptr [hInstance]
	.if (!EAX)
		invoke MsgBoxProc, NULL, addr szDialogFailed, addr szError,
				addr szClose, MB_OK
	.endif
	mov EAX, dword ptr [msg.wParam]
	ret
WinMain ENDP

;==============================================================================
;
; WndProc (hwnd, uMsg, wParam, lParam)
;
; Purpose:	The window procedure
;
; Entry:	hwnd	=	Handle to current instance of the application.
;			uMsg	=	Handle to app previous instance.
;			wParam	=	Pointer to a null terminated string specifying
;						app command line.
;			lParam	=	Specifies how the window is to be shown.
;
; Error Handling:	= 	Quit the program if the main window could not be
;						registered or when the main dialog box could not 
;						be initialized.	In any case it shows a message box 
;						with a descriptive message of the error.	
;
; Return:	TRUE	=	A message was processed.
;			FALSE	=	No message was processed;
;					 	Window class registration failed;
;					 	The main dialog box could not be initialized. 
;==============================================================================
	
WndProc PROC hwnd:HWND, uMsg:UINT, wParam:WPARAM, lParam:LPARAM
LOCAL pt:POINT
	
	.if dword ptr [uMsg]  == WM_INITDIALOG
		
	; --> Initialize brInfo STRUCTURE. <--
		
		push dword ptr [hwnd]
		pop dword ptr [brInfo.hwndOwner]
		
	; --> Get ComboBox handler. <--
		
		invoke GetDlgItem, dword ptr [hwnd], CBO_SWITCH
		mov dword ptr [hCombo], EAX
		invoke StartCombo, dword ptr [hwnd]
		
	.elseif dword ptr [uMsg] ==  WM_COMMAND
		
	; --> Menu commands. <--
		
		LOWORD (wParam)
		.if dword ptr [lParam] == 0
			.if EAX  == IDM_ABOUT
				invoke DialogBoxParam, dword ptr [hInstance],addr szDlgAbout, 
						dword ptr [hwnd], addr DlgProc, 0
	; --> Choose file to be scanned. <--
			.elseif EAX == IDM_CHOOSE_FILE || EAX == VK_IDM_CHOOSE_FILE
				invoke ChooseFile, dword ptr [hwnd]
				.if EAX	!= 0		; Error choosing file ?
					xor EAX, EAX
					ret
					
	; --> Show selected filename and its path. <--
					
				.else
					invoke SetDlgItemText, dword ptr [hwnd], EDT_1, 
							addr lpstrFile
					invoke SetDlgItemText, dword ptr [hwnd], EDT_2,
							addr szOutName
				.endif
				
	; --> Open Scan.ini for text edition. <--			
				
			.elseif EAX == IDM_EDIT_SCAN_INI || EAX == VK_IDM_EDIT_SCAN_INI
				invoke LoadIniFile, dword ptr [hwnd]
				.if EAX != 0
				    xor EAX, EAX
				    ret
				.endif
				invoke OpenToRead, dword ptr[hwnd], addr szScanini
				
	; --> Quit program. <--
				
			.elseif EAX == IDM_EXIT	|| EAX == VK_IDM_EXIT 
				invoke DestroyWindow, dword ptr [hwnd] 
				
	; --> Open Readme.txt. <--
				
			.elseif EAX == IDM_READ_ME || EAX == VK_IDM_READ_ME
				invoke OpenToRead, dword ptr [hwnd], addr szReadMe
				
	; -->  Reset main dialog controls. <--
				
			.elseif EAX == IDM_RESET || EAX == VK_IDM_RESET
				invoke Clean, dword ptr [hwnd]
				
	; --> Set path for Scan.exe <--
				
			.elseif EAX == IDM_SCAN_PATH || EAX == VK_IDM_SCAN_PATH
			    invoke SetScanPath,dword ptr [hwnd]
			
			.endif
			
	; --> Menu messages end. <--
			
		.else
			push EAX
			HIWORD (wParam)
			xchg EAX, EDX
			pop EAX
			
			.if EDX == BN_CLICKED
				
				.if EAX == BTN_CHOOSE
					invoke SendMessage,	dword ptr [hwnd], WM_COMMAND, 
							IDM_CHOOSE_FILE, NULL
					
				.elseif EAX == BTN_EXIT
					invoke SendMessage,	dword ptr [hwnd], WM_COMMAND,
							IDM_EXIT, NULL
					
	; --> Generates the .imp extension. <--
					
				.elseif EAX == BTN_GEN	
					invoke Verify, dword ptr [hwnd]
					
					.if EAX != 0		; Is there any operational error ?
						xor EAX, EAX
						ret
						
					.else
						invoke MontaLinha, dword ptr [hwnd]
						
						.if EAX != 0	
							invoke Clean, dword ptr [hwnd]
							xor EAX, EAX
							ret
						.endif 
						
	; --> All parameters correct. Run scan.exe. <--
						invoke Executa, dword ptr [hwnd], EDI
						
					.endif
				.endif
			.endif
		.endif
		
	.elseif dword ptr [uMsg] == WM_CONTEXTMENU
		mov		eax, dword ptr [wParam]
		.if eax==dword ptr [hwnd]
			mov	eax, dword ptr [lParam]
			push eax
			and	eax, 0FFFFh
			mov	dword ptr [pt.x], eax
			pop eax
			shr	eax, 16
			mov	dword ptr [pt.y], eax
			invoke GetMenu, dword ptr [hwnd]
			invoke GetSubMenu, EAX, 1
			invoke TrackPopupMenu, EAX, TPM_LEFTALIGN or TPM_RIGHTBUTTON,
					dword ptr [pt.x], dword ptr [pt.y], 0, 
					dword ptr [hwnd], 0
		.endif
		
	.elseif dword ptr [uMsg] ==  WM_DESTROY
		invoke PostQuitMessage, 0
		
	.elseif dword ptr [uMsg] == WM_CLOSE
		invoke DestroyWindow, hwnd 
		
	.else
		invoke DefWindowProc, dword ptr [hwnd],	dword ptr [uMsg], 
		        dword ptr [wParam],	dword ptr [lParam]
		ret
		
	.endif
	
	xor EAX, EAX
	ret
	
WndProc endp	
	
end start
;==============================================================================
;==                                 EOF                                      ==
;==============================================================================