TITLE Verify.asm
;                           =====================
;============================      Verify.asm   ===============================
;==                         =====================                            ==
;==                                                                          ==
;== Project         : ScanGui                                                ==
;== Filename        : Verify.asm                                             ==
;== Version         : 1                                                      ==
;== Date            : Jul 15, 2008                                           ==
;== Compiler        : Masm v. 6.14 by Microsoft Corporation.                 ==
;== Purpose         : Check parameters to send a correct command             ==
;==                   line to scan, then creates this line.                  ==
;==                                                                          ==
;==============================================================================
;==                                                                          ==
;== Copyright (c) 2008, P. H. Duarte                                         ==
;== All rights reserved.                                                     ==
;==                                                                          ==
;==============================================================================

; VERSION HISTORY:
; ----------------

; Version           : 1
; Date              : Jul 15, 2008
; Revised by        : P. H. Duarte
; Description       : Original Version.
;==============================================================================
; Version           : 2.01
; Date              : Nov 15, 2008
; Revised by        : P. H. Duarte
; Description       : * Implemented the use of settings.ini file.
;                     * Added safe strings functions on strsafe.inc 
;                       and strsafe.lib.
;					  * Added error checking for string concatanation.
;
;==============================================================================

.686
.model flat, stdcall
option casemap:none

;==============================================================================
;==                        Include Files                                     ==
;==============================================================================

include ScanGII.inc

;==============================================================================
;==                           Local Data                                     ==
;==============================================================================

.data
szNotOutputName		CHAR	"Give a valid name for output file.", 0
FindData    WIN32_FIND_DATA <FILE_ATTRIBUTE_ARCHIVE>
szOutFormat			CHAR	"  -f -f2  -g  -a -dc -ic -go -g2  -m -m2 -m3 -m4",
                            " -m5 -m6 -m7  -t -l2 -l3 -na  -h  -p", 0
szOptUnicode		CHAR	" -w ", 0
szOptCase			CHAR	" -is", 0
szNoWarnings		CHAR	" -n", 0
szOptName			CHAR	" -o:", 0
szScan				CHAR	"scan.exe ", 0         ; Program to run.
szSpace				CHAR	" ", 0
szBufferOverflow	CHAR	"The resulting string is very large.", 0
szScanCmdLine		CHAR	2 * MAX_PATH	DUP(0) 
szFilePath			CHAR	    MAX_PATH	DUP(0)
bOptions			BOOLEAN	3				DUP(0) ; Other options.

;==============================================================================
.code	

;==============================================================================
;
; Montalinha (hwnd)
;
; Purpose:	Create the command line to Scan.exe
;
; Entry:	hwnd	= Handle to main window.	
;
; Return:   EDI: Command line for scan.exe.
;           0:  Command line is ok.
;           1:  Command line is not safe.
;
; Error handling: Verify string operations error such as buffer overflow and 
;                 show a message box which tells the user about this error.
;
;==============================================================================

MontaLinha	proc hwnd:HWND
	
	push OFFSET szScanCmdLine
	pop EDI	; EDI -> Command line for scan.exe
	
	push EDI			
	invoke RtlZeroMemory, EDI, LENGTHOF szScanCmdLine ; Clean the command line.
	pop EDI
	
	invoke lstrlen, addr szScanPath		; Verify scan.exe path.
	.if EAX != 0						; Is it set ?
		; --> Concatenate path to command line. <--
		invoke StringCbCat, EDI, LENGTHOF szScanCmdLine, addr szScanPath
		.if EAX != S_OK
			invoke MsgBoxProc, dword ptr [hwnd], addr szBufferOverflow, 
					addr szError, addr szClose, MB_OK
			mov EAX, 1
			ret
		.endif

	.endif

		; --> Concatenate scan.exe to command line. <--
	invoke StringCbCat, EDI, LENGTHOF szScanCmdLine, addr szScan
	.if EAX != S_OK
		invoke MsgBoxProc, dword ptr [hwnd], addr szBufferOverflow, 
				addr szError, addr szClose, MB_OK
		mov EAX, 1
		ret
	.endif

	invoke StringCbCat, EDI, LENGTHOF szScanCmdLine, addr szSpace
	.if EAX != S_OK
		invoke MsgBoxProc, dword ptr [hwnd], addr szBufferOverflow, 
				addr szError, addr szClose, MB_OK
		mov EAX, 1
		ret
	.endif	
	
	; --> Concatenate the file to scan. <--	
	invoke StringCbCat, EDI, LENGTHOF szScanCmdLine, addr lpstrFile
	.if EAX != S_OK
		invoke MsgBoxProc, dword ptr [hwnd], addr szBufferOverflow, 
				addr szError, addr szClose, MB_OK
		mov EAX, 1
		ret
	.endif
		
	; --> Choosed compilator. <-- 
	invoke SendMessage, dword ptr [hCombo], CB_GETCURSEL, NULL, NULL  
	
	shl EAX, 02											 
	add EAX, OFFSET szOutFormat	; EAX -> Compiler switch. 
	xchg EAX, ESI				; ESI -> Compiler string.
	
	invoke lstrlen, EDI
	push EDI
	add EDI, EAX
	mov ECX, 04 
	cld
	rep movsb
	pop EDI
	
	; --> Add source file name <-- 
	
	for concaten, <szOptName, szFilePath> 
	invoke StringCbCat, EDI, LENGTHOF szScanCmdLine, addr concaten
	.if EAX != S_OK
		invoke MsgBoxProc, dword ptr [hwnd], addr szBufferOverflow, 
				addr szError, addr szClose, MB_OK
		mov EAX, 1
		ret
	.endif
	endm
	
	; --> Add importations file name <-- 
	
	invoke GetDlgItemText, dword ptr [hwnd], EDT_2, addr szOutName, MAX_PATH
	
	invoke StringCbCat, EDI, LENGTHOF szScanCmdLine, addr szOutName
	.if EAX != S_OK
		invoke MsgBoxProc, dword ptr [hwnd], addr szBufferOverflow, 
				addr szError, addr szClose, MB_OK
		mov EAX, 1
		ret
	.endif	
	
	; --> Add other options ( Unicode, No warnings ... ). <-- 	
	
	optpointer = 0
	for concateno, <szOptUnicode, szOptCase, szNoWarnings>
	.if byte ptr ds:[bOptions + optpointer] == 1
		invoke StringCbCat, EDI, LENGTHOF szScanCmdLine, addr concateno
		.if EAX != S_OK
			invoke MsgBoxProc, dword ptr [hwnd], addr szBufferOverflow, 
					addr szError, addr szClose, MB_OK
			mov EAX, 1
			ret
		.endif
	.endif
	optpointer = optpointer + 1
	endm
	
	xor EAX, EAX ; No errors. Command line is safe.
	ret
MontaLinha	endp

;==============================================================================
;
; Function Verify (hwnd )
;
; Purpose: Verify parameters to be pass in command line for scan.exe  
;
; Entry:		hwnd	= Handle to main window.
;
; Called by:    WndProc 
;
;Description:	This function searches for '\' character in the source
;               file name from the end of the string to its 'zero point'.
;               It doesn't verify if any file exists !!! It still checks
;               and sets the correct extra options in the check box controls of 
;				program.  
;
; Returns:		0:	 No errors.
;               1:   An error occured.		
;
; Error Handling: Verifies if a valid name and valid path exists for
;                 source file. Then it does the same for the imports file
;                 name. If there is an error, shows a message box which
;                 explain to user the fact and asks for valid path and valid
;                 name for both source and imports file.
;==============================================================================

Verify PROC hwnd:HWND
	
	
	; --> Verify source file name. <-- 
	invoke GetDlgItemText, dword ptr [hwnd], EDT_1, addr lpstrFile, MAX_PATH
	
	.if EAX == 0 ; Source file not selected.
		invoke MsgBoxProc, dword ptr [hwnd], addr szHaveNotSelected,
		        addr szError, addr szClose, MB_ICONSTOP
		mov EAX, 1
		ret
		
	; --> Verify valid file name. <-- 
	; --> File name must have a '\' in its string. <-- 
	.else	
		xchg EAX, ECX			; ECX -> Lenght of file name string.
		push OFFSET lpstrFile          	
		pop EDI					; EDI -> Start point of string.
		add EDI, ECX			; EDI -> End point of string.
		std						      
		and ECX, 0ffffh			; ECX -> Lenght of lpstrFile.
		mov al, '\'				; Search for '\' in file name.
		and EAX, 0ffh                   	
		repne scasb                     	
		or ECX, ECX                     	
		jnz @f
	; -->The file name is invalid. <--
		invoke MsgBoxProc, dword ptr [hwnd], addr szInvalidName,
		        addr szError, addr szClose, MB_OK					        	
		mov EAX, 1
		ret
@@:
		inc ECX
		inc ECX
		push OFFSET lpstrFile
		pop ESI
		mov EDI, OFFSET szFilePath
		cld						; Step forward
		rep movsb
		
		
		invoke GetDlgItemText, dword ptr [hwnd], EDT_2, addr  szOutName, MAX_PATH
		.if EAX == 0	; No name for imports file.
			invoke MsgBoxProc, dword ptr [hwnd], addr szNotOutputName,
			addr szError, addr szClose, MB_OK
	        mov EAX, 1
			ret          	
		.endif                          	
		
	; --> Verify check buttons. <--            	
		mov EBX, CHK_1 			; EBX -> Check buttons control counter.
		.while EBX <= CHK_3             	
			push EBX                    	
			invoke GetDlgItem, dword ptr [hwnd], EBX
			mov dword ptr [hCheck], EAX
			invoke SendMessage, dword ptr [hCheck], BM_GETCHECK, NULL, NULL 
			and EAX, 00ffh 
			sub EBX, CHK_1
			mov byte ptr ds:[bOptions + EBX], AL
			pop EBX                             	
			inc EBX
		.endw
	.endif
	
	; --> Verify scan.exe path. <--
	invoke lstrlen, addr szScanPath	; Verify scan.exe path.
	.if EAX == 0					; Is it not set ?
		; --> Search for the file Settings.ini. <--
		invoke FindFirstFile, addr szIniFile, addr FindData
		.if EAX != INVALID_HANDLE_VALUE ; Does it exist ?
			invoke FindClose, EAX
			invoke LoadIniFile, dword ptr [hwnd]
			.if EAX != 0 ; It gives an error.
				jmp @F
			.else
				ret     ; File found and scan.exe path is ok.
			.endif   
    
        .elseif EAX == INVALID_HANDLE_VALUE
        ; --> Does scan.exe share the same folder as scangii.exe ? <--
@@: 		invoke FindFirstFile, addr szScan, addr FindData
			.if EAX == INVALID_HANDLE_VALUE
				invoke MsgBoxProc, dword ptr [hwnd], addr szSetPathFirst, 
				        addr szError, addr szClose, MB_ICONINFORMATION
				mov EAX, 1
				ret
			.else
				invoke FindClose, EAX ; Scan.exe and scangii.exe are
				                      ; at the same folder.
			.endif
			
		.endif
		
	.endif
	
	xor EAX, EAX
	ret
Verify endp			
end
;==============================================================================
;==                                 EOF                                      ==
;==============================================================================